/**
 * @module package/quiqqer/crm/bin/backend/classes/TaskStatus
 * @author www.pcsg.de (Henning Leutz)
 */
define('package/quiqqer/crm/bin/backend/classes/TaskStatus', [

    'qui/QUI',
    'qui/classes/DOM',
    'Ajax'

], function (QUI, QUIDOM, QUIAjax) {
    "use strict";

    return new Class({

        Extends: QUIDOM,
        Type   : 'package/quiqqer/crm/bin/backend/classes/TaskStatus',

        initialize: function (options) {
            this.parent(options);
        },

        /**
         * Return the task status list for a grid
         *
         * @return {Promise}
         */
        getList: function () {
            return new Promise(function (resolve, reject) {
                QUIAjax.get('package_quiqqer_crm_ajax_backend_taskStatus_list', resolve, {
                    'package': 'quiqqer/crm',
                    onError  : reject
                });
            });
        },

        /**
         * Return next available ID
         *
         * @return {Promise}
         */
        getNextId: function () {
            return new Promise(function (resolve, reject) {
                QUIAjax.get('package_quiqqer_crm_ajax_backend_taskStatus_getNextId', resolve, {
                    'package': 'quiqqer/crm',
                    onError  : reject
                });
            });
        },

        /**
         * Create a new task status
         *
         * @param {String|Number} id - Task Status ID
         * @param {String} color
         * @param {Object} title - {de: '', en: ''}
         * @return {Promise}
         */
        createTaskStatus: function (id, color, title) {
            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_crm_ajax_backend_taskStatus_create', function (result) {
                    require([
                        'package/quiqqer/translator/bin/Translator'
                    ], function (Translator) {
                        Translator.refreshLocale().then(function () {
                            resolve(result);
                        });
                    });
                }, {
                    'package': 'quiqqer/crm',
                    id       : id,
                    color    : color,
                    title    : JSON.encode(title),
                    onError  : reject
                });
            });
        },

        /**
         * Delete a task status
         *
         * @param {String|Number} id - Task Status ID
         * @return {Promise}
         */
        deleteTaskStatus: function (id) {
            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_crm_ajax_backend_taskStatus_delete', function () {
                    require([
                        'package/quiqqer/translator/bin/Translator'
                    ], function (Translator) {
                        Translator.refreshLocale().then(function () {
                            resolve();
                        });
                    });
                }, {
                    'package': 'quiqqer/crm',
                    id       : id,
                    onError  : reject
                });
            });
        },

        /**
         * Return the status data
         *
         * @param {String|Number} id - Task Status ID
         * @return {Promise}
         */
        getTaskStatus: function (id) {
            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_crm_ajax_backend_taskStatus_get', resolve, {
                    'package': 'quiqqer/crm',
                    id       : id,
                    onError  : reject
                });
            });
        },

        /**
         * Return the status data
         *
         * @param {String|Number} id - Task Status ID
         * @param {String} color
         * @param {Object} title - {de: '', en: ''}
         * @return {Promise}
         */
        updateTaskStatus: function (id, color, title) {
            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_crm_ajax_backend_taskStatus_update', resolve, {
                    'package': 'quiqqer/crm',
                    id       : id,
                    color    : color,
                    title    : JSON.encode(title),
                    onError  : reject
                });
            });
        }
    });
});
