/**
 * @module package/quiqqer/crm/bin/backend/controls/TaskList
 * @author www.pcsg.de (Henning Leutz)
 */
define('package/quiqqer/crm/bin/backend/controls/TaskList', [

    'qui/QUI',
    'qui/controls/Control',
    'controls/grid/Grid',
    'Locale',
    'Ajax',

    'css!package/quiqqer/crm/bin/backend/controls/TaskList.css'

], function (QUI, QUIControl, Grid, QUILocale, QUIAjax) {
    "use strict";

    var lg = 'quiqqer/crm';

    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/crm/bin/backend/controls/TaskList',

        Binds: [
            '$onImport',
            'refresh',
            'openAddWindow',
            'openDeleteWindow',
            '$onDblClick'
        ],

        options: {
            page   : 1,
            perPage: 50,
            userId : false
        },

        initialize: function (options) {
            this.parent(options);

            this.$Container = null;
            this.$Panel     = null;

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * event: on import
         */
        $onImport: function () {
            this.getElm().addClass('quiqqer-crm-create-task-list');

            var Table = this.getElm().getParent('table');
            var Form  = Table.getParent('form');
            var Panel = this.getElm().getParent('.qui-panel');

            if (Panel) {
                this.$Panel = QUI.Controls.getById(Panel.get('data-quiid'));
            }

            if (!this.getAttribute('userId')) {
                this.setAttribute('userId', this.$Panel.getAttribute('userId'));
            }

            Table.setStyle('display', 'none');
            Form.setStyle('height', '100%');

            this.$Container = new Element('div', {
                styles: {
                    height: '100%',
                    width : '100%'
                }
            }).inject(Form);

            var columnModel = [{
                header   : '',
                dataIndex: 'issuableDisplay',
                dataType : 'node',
                width    : 60,
                className: 'grid-align-center'
            }, {
                header   : QUILocale.get('quiqqer/quiqqer', 'status'),
                dataIndex: 'statusDisplay',
                dataType : 'html',
                width    : 100,
                className: 'grid-align-center'
            }, {
                header   : QUILocale.get(lg, 'panel.tasks.grid.to'),
                dataIndex: 'toDisplay',
                dataType : 'string',
                width    : 200
            }, {
                header   : QUILocale.get(lg, 'panel.tasks.grid.type'),
                dataIndex: 'type',
                dataType : 'string',
                width    : 100,
                className: 'grid-align-center'
            }, {
                header   : QUILocale.get(lg, 'panel.tasks.grid.title'),
                dataIndex: 'title',
                dataType : 'string',
                width    : 500
            }];


            this.$Grid = new Grid(this.$Container, {
                buttons: [{
                    name     : 'add',
                    textimage: 'fa fa-plus',
                    text     : QUILocale.get(lg, 'crm.task.list.add.button'),
                    events   : {
                        onClick: this.openAddWindow
                    }
                }, {
                    name     : 'delete',
                    textimage: 'fa fa-trash',
                    text     : QUILocale.get(lg, 'crm.task.list.delete.button'),
                    disabled : true,
                    events   : {
                        onClick: this.openDeleteWindow
                    }
                }],

                columnModel      : columnModel,
                pagination       : true,
                filterInput      : true,
                perPage          : this.getAttribute('perPage'),
                page             : this.getAttribute('page'),
                sortOn           : this.getAttribute('field'),
                serverSort       : true,
                showHeader       : true,
                sortHeader       : true,
                alternaterows    : true,
                resizeColumns    : true,
                selectable       : true,
                multipleSelection: true,
                resizeHeaderOnly : true,
                editable         : true,
                editondblclick   : true
            });

            this.$Grid.addEvents({
                onRefresh : this.refresh,
                onDblClick: this.$onDblClick
            });

            this.resize();
            this.refresh();
        },

        /**
         * refresh the list
         */
        refresh: function () {
            var self = this;

            this.fireEvent('refreshBegin', [this]);

            QUIAjax.get('package_quiqqer_crm_ajax_backend_tasks_grid_getByCustomer', function (result) {
                var i, len, entry;

                for (i = 0, len = result.data.length; i < len; i++) {
                    entry = result.data[i];

                    if (entry.issuable) {
                        result.data[i].issuableDisplay = new Element('div', {
                            'class': 'fa fa-clock-o'
                        });
                    } else {
                        result.data[i].issuableDisplay = new Element('div', {
                            html: '&nbsp;'
                        });
                    }

                    result.data[i].attachment = new Element('div', {
                        html: '&nbsp;'
                    });

                    if (entry.from === '' || !entry.from) {
                        result.data[i].fromDisplay = '---';
                    } else {
                        result.data[i].fromDisplay = result.data[i].from;
                    }

                    if (entry.to === '' || !entry.to) {
                        result.data[i].toDisplay = '---';
                    } else {
                        result.data[i].toDisplay = result.data[i].to;
                    }
                }

                self.$Grid.setData(result);
                self.fireEvent('refreshEnd', [self]);
            }, {
                'package' : 'quiqqer/crm',
                customerId: this.getAttribute('userId'),
                search    : '',
                options   : JSON.encode({
                    perPage: this.$Grid.options.perPage,
                    page   : this.$Grid.options.page,
                    sortBy : this.$Grid.options.sortBy,
                    sortOn : this.$Grid.options.sortOn
                }),
                filter    : JSON.encode({})
            });
        },

        /**
         * Resize the control
         */
        resize: function () {
            if (!this.$Grid) {
                return Promise.resolve();
            }

            var size = this.$Container.getSize();

            return Promise.all([
                this.$Grid.setHeight(size.y),
                this.$Grid.setWidth(size.x)
            ]);
        },

        /**
         * open the add window
         */
        openAddWindow: function () {
            var self       = this;
            var customerId = false;

            if (this.$Panel) {
                customerId = this.$Panel.getAttribute('userId');
            }

            require([
                'package/quiqqer/crm/bin/backend/controls/CreateTaskWindow'
            ], function (CreateTaskWindow) {
                new CreateTaskWindow({
                    customerId        : customerId,
                    customerChangeable: !customerId,
                    events            : {
                        onSubmit: self.refresh
                    }
                }).open();
            });
        },

        /**
         * open the delete window
         */
        openDeleteWindow: function () {

        },


        //region events

        /**
         * event: on dbl click
         *
         * @param event
         */
        $onDblClick: function (event) {
            var self = this,
                row  = this.$Grid.getDataByRow(event.row);

            this.fireEvent('editBegin', [this]);

            require([
                'package/quiqqer/crm/bin/backend/controls/EditTaskWindow'
            ], function (EditTaskWindow) {
                new EditTaskWindow({
                    taskId: row.id,
                    events: {
                        onSubmit: self.refresh
                    }
                }).open();

                self.fireEvent('editEnd', [self]);
            });
        }

        //endregion
    });
});
