/**
 * @module package/quiqqer/crm/bin/backend/controls/TaskPanel
 * @author www.pcsg.de (Henning Leutz)
 */
define('package/quiqqer/crm/bin/backend/controls/TaskPanel', [

    'qui/QUI',
    'qui/controls/desktop/Panel',
    'package/quiqqer/calendar-controls/bin/controls/TimeFilter',
    'controls/grid/Grid',
    'Locale',
    'Ajax',

    'css!package/quiqqer/crm/bin/backend/controls/TaskList.css'

], function (QUI, QUIPanel, TimeFilter, Grid, QUILocale, QUIAjax) {
    "use strict";

    var lg = 'quiqqer/crm';

    return new Class({

        Extends: QUIPanel,
        Type   : 'package/quiqqer/crm/bin/backend/controls/TaskPanel',

        Binds: [
            'resize',
            'refresh',
            'openAddWindow',
            'openDeleteWindow',
            '$filterIssuableClick',
            '$onDblClick',
            '$onCreate',
            '$onSearchKeyUp'
        ],

        options: {
            search: false
        },

        initialize: function (options) {
            this.parent(options);

            this.$Container  = null;
            this.$Grid       = null;
            this.$TimeFilter = null;
            this.$Search     = null;

            this.$issuable = true; // true = only show issuable

            this.setAttributes({
                title  : QUILocale.get(lg, 'panel.tasks.title'),
                perPage: 50,
                page   : 1
            });

            this.addEvents({
                onCreate: this.$onCreate
            });
        },

        /**
         * event: on create
         */
        $onCreate: function () {
            this.addButton({
                name  : 'issuable',
                icon  : 'fa fa-clock-o',
                title : QUILocale.get(lg, 'crm.task.list.issuable.button.title'),
                events: {
                    onClick: this.$filterIssuableClick
                }
            });

            this.getButtons('issuable').setActive();

            // buttons
            this.$TimeFilter = new TimeFilter({
                name  : 'timeFilter',
                styles: {
                    'float': 'right'
                },
                events: {
                    onChange: this.refresh
                }
            });

            this.addButton(this.$TimeFilter);

            this.addButton({
                type  : 'separator',
                styles: {
                    'float': 'right'
                }
            });

            this.addButton({
                name  : 'search',
                icon  : 'fa fa-search',
                styles: {
                    'float': 'right'
                },
                events: {
                    onClick: this.refresh
                }
            });

            this.$Search = new Element('input', {
                placeholder: QUILocale.get(lg, 'crm.task.list.search.placeholder'),
                styles     : {
                    'float': 'right',
                    margin : '10px 0 0 0',
                    width  : 200
                },
                events     : {
                    keyup: this.$onSearchKeyUp
                }
            });

            if (this.getAttribute('search')) {
                this.$Search.value = this.getAttribute('search');
            }

            this.addButton(this.$Search);


            // grid
            this.$Container = new Element('div', {
                styles: {
                    height: '100%',
                    width : '100%'
                }
            }).inject(this.getContent());

            this.getContent().setStyle('height', 'calc(100% -  90px)');
            this.getContent().setStyle('padding', 10);

            var columnModel = [{
                header   : '',
                dataIndex: 'issuableDisplay',
                dataType : 'node',
                width    : 60,
                className: 'grid-align-center'
            }, {
                header   : QUILocale.get('quiqqer/quiqqer', 'status'),
                dataIndex: 'statusDisplay',
                dataType : 'html',
                width    : 100,
                className: 'grid-align-center'
            }, {
                header   : QUILocale.get(lg, 'panel.tasks.grid.to'),
                dataIndex: 'toDisplay',
                dataType : 'string',
                width    : 200
            }, {
                header   : QUILocale.get(lg, 'panel.tasks.grid.customer'),
                dataIndex: 'customerDisplay',
                dataType : 'string',
                width    : 200
            }, {
                header   : QUILocale.get(lg, 'panel.tasks.grid.type'),
                dataIndex: 'type',
                dataType : 'string',
                width    : 100,
                className: 'grid-align-center'
            }, {
                header   : QUILocale.get(lg, 'panel.tasks.grid.title'),
                dataIndex: 'title',
                dataType : 'string',
                width    : 500
            }];


            this.$Grid = new Grid(this.$Container, {
                buttons: [{
                    name     : 'add',
                    textimage: 'fa fa-plus',
                    text     : QUILocale.get(lg, 'crm.task.list.add.button'),
                    events   : {
                        onClick: this.openAddWindow
                    }
                }, {
                    name     : 'delete',
                    textimage: 'fa fa-trash',
                    text     : QUILocale.get(lg, 'crm.task.list.delete.button'),
                    disabled : true,
                    events   : {
                        onClick: this.openDeleteWindow
                    }
                }],

                columnModel      : columnModel,
                pagination       : true,
                filterInput      : true,
                perPage          : this.getAttribute('perPage'),
                page             : this.getAttribute('page'),
                sortOn           : this.getAttribute('field'),
                serverSort       : true,
                showHeader       : true,
                sortHeader       : true,
                alternaterows    : true,
                resizeColumns    : true,
                selectable       : true,
                multipleSelection: true,
                resizeHeaderOnly : true,
                editable         : true,
                editondblclick   : true
            });

            this.$Grid.addEvents({
                onRefresh : this.refresh,
                onDblClick: this.$onDblClick
            });

            this.resize();
            this.refresh();
        },

        /**
         * resize the grid
         *
         * @return {Promise}
         */
        resize: function () {
            if (!this.$Grid) {
                return Promise.resolve();
            }

            var size = this.$Container.getSize();

            return Promise.all([
                this.$Grid.setHeight(size.y),
                this.$Grid.setWidth(size.x)
            ]);
        },

        /**
         * refresh the grid
         */
        refresh: function () {
            if (!this.$Search) {
                return;
            }

            var self = this;

            this.Loader.show();
            this.fireEvent('refreshBegin', [this]);

            QUIAjax.get('package_quiqqer_crm_ajax_backend_tasks_grid_search', function (result) {
                var i, len, entry;

                for (i = 0, len = result.data.length; i < len; i++) {
                    entry = result.data[i];

                    if (entry.issuable) {
                        result.data[i].issuableDisplay = new Element('div', {
                            'class': 'fa fa-clock-o'
                        });
                    } else {
                        result.data[i].issuableDisplay = new Element('div', {
                            html: '&nbsp;'
                        });
                    }

                    result.data[i].attachment = new Element('div', {
                        html: '&nbsp;'
                    });

                    if (entry.from === '' || !entry.from) {
                        result.data[i].fromDisplay = '---';
                    } else {
                        result.data[i].fromDisplay = result.data[i].from;
                    }

                    if (entry.to === '' || !entry.to) {
                        result.data[i].toDisplay = '---';
                    } else {
                        result.data[i].toDisplay = result.data[i].to;
                    }
                }

                self.$Grid.setData(result);
                self.Loader.hide();
                self.fireEvent('refreshEnd', [self]);
            }, {
                'package': 'quiqqer/crm',
                search   : this.$Search.value,
                options  : JSON.encode({
                    perPage : this.$Grid.options.perPage,
                    page    : this.$Grid.options.page,
                    sortBy  : this.$Grid.options.sortBy,
                    sortOn  : this.$Grid.options.sortOn,
                    issuable: this.$issuable ? 1 : 0
                }),
                filter   : JSON.encode({
                    from: [this.$TimeFilter.getValue().from, '>='],
                    to  : [this.$TimeFilter.getValue().to, '<=']
                })
            });
        },

        /**
         * filter the issuable tasks
         */
        $filterIssuableClick: function () {
            var Issuable = this.getButtons('issuable');

            if (Issuable.isActive()) {
                Issuable.setNormal();
                this.$issuable = false;
            } else {
                Issuable.setActive();
                this.$issuable = true;
            }

            this.refresh();
        },

        /**
         * event: on dbl click -> edit task
         */
        $onDblClick: function (event) {
            var self = this,
                data = this.$Grid.getDataByRow(event.row);

            this.fireEvent('editBegin', [this]);

            require([
                'package/quiqqer/crm/bin/backend/controls/EditTaskWindow'
            ], function (EditTaskWindow) {
                new EditTaskWindow({
                    taskId: data.id,
                    events: {
                        onSubmit: self.refresh
                    }
                }).open();

                self.fireEvent('editEnd', [self]);
            });
        },

        /**
         * open add window
         */
        openAddWindow: function () {
            var self = this;

            require([
                'package/quiqqer/crm/bin/backend/controls/CreateTaskWindow'
            ], function (CreateTaskWindow) {
                new CreateTaskWindow({
                    events: {
                        onSubmit: self.refresh
                    }
                }).open();
            });
        },

        /**
         * @todo
         */
        openDeleteWindow: function () {

        },

        /**
         * event: search key up
         * @param event
         */
        $onSearchKeyUp: function (event) {
            if (event.key === 'enter') {
                this.refresh();
            }
        }
    });
});
