<?php

namespace QUI\ERP\CRM;

use QUI;

/**
 * Class Factory
 *
 * @package QUI\ERP\CRM
 */
class TaskFactory
{
    /**
     * @param array $attributes
     * @param null|QUI\Interfaces\Users\User $User - optional
     *
     * @return Task
     *
     * @throws TaskException
     */
    public static function create(array $attributes = [], $User = null)
    {
        if ($User === null) {
            $User = QUI::getUserBySession();
        }

        $needles = [
            'title',
            'text',
            'customerId',
            'from',
            'to'
        ];

        foreach ($needles as $attribute) {
            if (!isset($attributes[$attribute])) {
                $attributes[$attribute] = '';
            }
        }

        if (empty($attributes['taskType'])) {
            $attributes['taskType'] = TaskHandler::TASK_TYPE_TASK;
        }

        if (empty($attributes['status'])) {
            $attributes['status'] = 0;
        }

        if (empty($attributes['from'])) {
            $attributes['from'] = \date('Y-m-d H:i:s');
        }

        if (empty($attributes['to'])) {
            $attributes['to'] = null;
        }

        // check customer id
        if (!empty($attributes['customerId'])) {
            try {
                $attributes['customerId'];
            } catch (QUI\Exception $Exception) {

            }
        }


        try {
            QUI::getDataBase()->insert(TaskHandler::table(), [
                'title'    => $attributes['title'],
                'text'     => $attributes['text'],
                'from'     => $attributes['from'],
                'to'       => $attributes['to'],
                'userId'   => $attributes['customerId'],
                'editorId' => $User->getId(),
                'taskType' => $attributes['taskType'],
                'status'   => $attributes['status']
            ]);
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::addAlert($Exception->getMessage());

            throw new TaskException(['quiqqer/erp', 'exception.task.error']);
        }

        $taskId = QUI::getDataBase()->getPDO()->lastInsertId();

        return TaskHandler::getTask($taskId);
    }
}
